# -*- coding: utf-8 -*-
"""Taxonomy utility functions."""

import six
from flask import current_app
from werkzeug.utils import import_string


def obj_or_import_string(value, default=None):
    """
    Import string or return object.
    :params value: Import path or class object to instantiate.
    :params default: Default object to return if the import fails.
    :returns: The imported object.
    """
    if isinstance(value, six.string_types):
        return import_string(value)
    elif value:  # pragma: nocover
        return value  # pragma: nocover
    return default  # pragma: nocover


def load_or_import_from_config(key, app=None, default=None):
    """
    Load or import value from config.
    :returns: The loaded value.
    """
    app = app or current_app
    imp = app.config.get(key)
    return obj_or_import_string(imp, default=default)
